const path = require('path')
const CopyPlugin = require('copy-webpack-plugin')
const MiniCssExtractPlugin = require("mini-css-extract-plugin")
const HtmlWebpackPlugin = require('html-webpack-plugin')
const srcDir = '../src/'

module.exports = {
  entry: {
    main: path.join(__dirname, srcDir + 'main.js'),
    index: path.join(__dirname, srcDir + 'style/index.scss')
  },
  output: {
    path: path.join(__dirname, '../dist'),
    filename: 'js/[name].js'
  },
  optimization: {
    splitChunks: {
      name: 'vendor',
      chunks: 'initial'
    }
  },
  module: {
    rules: [
      {
        test: /\.(scss|sass)$/,
        use: [
          {
            loader: MiniCssExtractPlugin.loader,
            options: {
              publicPath: '../',
            },
          },
          "css-loader",
          "sass-loader"
        ]
      },
      {
        test: /\.(png|jpe?g|gif)$/i,
        use: [
          {
            loader: 'file-loader',
            options: {
              name: `img/[name].[hash:8].[ext]`
            }
          },
        ],
      }
    ]
  },
  resolve: {
    extensions: ['.js']
  },
  plugins: [
    // new CopyPlugin({
    //   patterns: [{ from: '.', to: path.join(__dirname, '../dist'), context: 'public' }],
    //   options: {}
    // }),
    new MiniCssExtractPlugin({
      filename: "css/[name].css"
    }),
    new HtmlWebpackPlugin({
      template: path.join(__dirname, srcDir + 'index.html'),
      filename: 'index.html',
    }),
    new HtmlWebpackPlugin({
      template: path.join(__dirname, srcDir + '/template/home.html'),
      filename: 'home.html',
    }),
  ]
}
